#/usr/bin/env julia
ENV["MPLBACKEND"] = "Agg"
import Granular
import PyPlot
import ArgParse
using Random
import DelimitedFiles

let
render = false

function parse_command_line()
    s = ArgParse.ArgParseSettings()
    ArgParse.@add_arg_table s begin
        "--E"
            help = "Young's modulus [Pa]"
            arg_type = Float64
            default = 2e7
        "--nu"
            help = "Poisson's ratio [-]"
            arg_type = Float64
            default = 0.285
        "--mu_d"
            help = "dynamic friction coefficient [-]"
            arg_type = Float64
            default = 0.3
        "--tensile_strength"
            help = "the maximum tensile strength [Pa]"
            arg_type = Float64
            default = 400e3
        "--tensile_strength_std_dev"
            help = "standard deviation of the maximum tensile strength [Pa]"
            arg_type = Float64
            default = 0.0
        "--shear_strength"
            help = "the maximum shear strength [Pa]"
            arg_type = Float64
            default = 200e3
        "--fracture_toughness"
            help = "fracture toughness [Pa*m^{1/2}]"
            arg_type = Float64
            default = 1e20
            #default = 1285e3
        "--r_min"
            help = "min. grain radius [m]"
            arg_type = Float64
            default = 20.0
        "--r_max"
            help = "max. grain radius [m]"
            arg_type = Float64
            default = 80.0
        "--thickness"
            help = "grain thickness [m]"
            arg_type = Float64
            default = 1.
        "--rotating"
            help = "allow the grains to rotate"
            arg_type = Bool
            default = true
        "--compressive_velocity"
            help = "compressive velocity [m/s]"
            arg_type = Float64
            default = 0.1
        "--seed"
            help = "seed for the pseudo RNG"
            arg_type = Int
            default = 1
        "id"
            help = "simulation id"
            required = true
    end
    return ArgParse.parse_args(s)
end

function report_args(parsed_args)
    println("Parsed args:")
    for (arg,val) in parsed_args
        println("  $arg  =>  $val")
    end
end

function run_simulation(id::String,
                        E::Float64,
                        nu::Float64,
                        mu_d::Float64,
                        tensile_strength::Float64,
                        tensile_strength_std_dev::Float64,
                        shear_strength::Float64,
                        fracture_toughness::Float64,
                        r_min::Float64,
                        r_max::Float64,
                        thickness::Float64,
                        rotating::Bool,
                        compressive_velocity::Float64,
                        seed::Int)

    ############################################################################
    #### SIMULATION PARAMETERS
    ############################################################################

    # Grain mechanical properties
    youngs_modulus = E              # Elastic modulus [Pa]
    poissons_ratio = nu             # Shear-stiffness ratio [-]
    contact_dynamic_friction = mu_d # Coulomb-frictional coefficient [-]

    # Simulation duration [s]
    t_total = 60.0 * 60.0 * 2.0

    # Temporal interval between output files
    file_dt = 18.0

    ############################################################################
    #### Read prior state
    ############################################################################
	sim = Granular.readSimulation("rotating01-seed1/rotating01-seed1.681.jld2")
	sim.id = id
	Granular.resetTime!(sim)
	Granular.zeroKinematics!(sim)
    Granular.removeSimulationFiles(sim)

    # Set grain mechanical properties
    for grain in sim.grains
        grain.youngs_modulus = youngs_modulus
        grain.poissons_ratio = poissons_ratio
        grain.tensile_strength = abs(tensile_strength +
                                     randn()*tensile_strength_std_dev)
        grain.shear_strength = abs(shear_strength +
                                   randn()*tensile_strength_std_dev)
        grain.contact_static_friction = contact_dynamic_friction  # mu_s = mu_d
        grain.contact_dynamic_friction = contact_dynamic_friction
        grain.fracture_toughness = fracture_toughness
        grain.rotating = rotating
        grain.ocean_drag_coeff_horiz = 1.6e-4
        grain.ocean_drag_coeff_vert = 0.14
    end

	x_max = -Inf
	x_min = +Inf
	y_max = -Inf
	y_min = +Inf
	r_max = 0.0
	for grain in sim.grains
		r = grain.contact_radius

		if x_min > grain.lin_pos[1] - r
			x_min = grain.lin_pos[1] - r
		end

		if x_max < grain.lin_pos[1] + r
			x_max = grain.lin_pos[1] + r
		end

		if y_min > grain.lin_pos[2] - r
			y_min = grain.lin_pos[2] - r
		end

		if y_max < grain.lin_pos[2] + r
			y_max = grain.lin_pos[2] + r
		end
		
		if r_max < r
			r_max = r
		end
	end

	dx = r_max * 2.0
	L::Vector{Float64} = [(x_max - x_min)*3.0, y_max - y_min, dx]
	n = convert(Vector{Int}, floor.(L./dx))
	sim.ocean = Granular.createRegularOceanGrid(n, L,
                                                origo=[x_min - L[1]/3.0, y_min],
                                                time=[0.], name="resized")
    Granular.setGridBoundaryConditions!(sim.ocean, "impermeable", "-x +x")
    Granular.setGridBoundaryConditions!(sim.ocean, "impermeable", "-y +y")

	sim.walls[1].bc = "velocity"
	sim.walls[1].vel = -(y_max - y_min) * 0.2 / t_total

	# fix grains adjacent to walls to create no-slip BC
	fixed_thickness = 2.0 * r_max
	for grain in sim.grains
		if grain.lin_pos[2] <= y_min + fixed_thickness
			grain.fixed = true
			grain.color = 1
		elseif grain.lin_pos[2] >= y_max - fixed_thickness
			grain.allow_y_acc = true
			grain.fixed = true
			grain.color = 1
		end
	end

    Granular.setTimeStep!(sim)
    Granular.setTotalTime!(sim, t_total)
    Granular.setOutputFileInterval!(sim, file_dt)
    render && Granular.plotGrains(sim, verbose=true)

    ############################################################################
    #### RUN THE SIMULATION
    ############################################################################

	# Run the consolidation experiment, and monitor top wall position over
	# time
	time = Float64[]
	compaction = Float64[]
	effective_normal_stress = Float64[]
	while sim.time < sim.time_total

		for i=1:100
			Granular.run!(sim, single_step=true)
		end

		append!(time, sim.time)
		append!(compaction, sim.walls[1].pos)
		append!(effective_normal_stress, Granular.getWallNormalStress(sim))

	end
	Granular.writeSimulation(sim)

	defined_normal_stress = ones(length(effective_normal_stress)) *
	Granular.getWallNormalStress(sim, stress_type="effective")
	PyPlot.subplot(211)
	PyPlot.subplots_adjust(hspace=0.0)
	ax1 = PyPlot.gca()
	PyPlot.setp(ax1[:get_xticklabels](),visible=false) # Disable x labels
	PyPlot.plot(time, compaction)
	PyPlot.ylabel("Top wall height [m]")
	PyPlot.subplot(212, sharex=ax1)
	PyPlot.plot(time, defined_normal_stress)
	PyPlot.plot(time, effective_normal_stress)
	PyPlot.xlabel("Time [s]")
	PyPlot.ylabel("Normal stress [Pa]")
	PyPlot.savefig(sim.id * "-time_vs_compaction-stress.pdf")
	PyPlot.clf()
	DelimitedFiles.writedlm(sim.id * "-data.txt", [time, compaction,
									effective_normal_stress,
									defined_normal_stress])

    render && Granular.render(sim, trim=false, animation=true)
end

function main()
    parsed_args = parse_command_line()
    report_args(parsed_args)

    seed = parsed_args["seed"]
    run_simulation(parsed_args["id"] * "-seed" * string(seed),
                   parsed_args["E"],
                   parsed_args["nu"],
                   parsed_args["mu_d"],
                   parsed_args["tensile_strength"],
                   parsed_args["tensile_strength_std_dev"],
                   parsed_args["shear_strength"],
                   parsed_args["fracture_toughness"],
                   parsed_args["r_min"],
                   parsed_args["r_max"],
                   parsed_args["thickness"],
                   parsed_args["rotating"],
                   parsed_args["compressive_velocity"],
                   seed)
end

main()
end
